package br.com.zemaria.database;

import java.sql.*;
import java.io.*;
import java.util.*;

/**
 * Classe representando uma estratgia simplificada de acesso a banco de dados
 * via JDBC, um aspecto importante nessa classe  utilizao de um arquivo
 * contendo as definies de : driver, url, usurio, senha e comandos sql num
 * arquivo texto. O qual deve seguir a seguinte a seguinte estrutura: <br>
 * [DRIVER]  com.inet.tds.TdsDriver <br>
 * [URL]  jdbc:inetdae:localhost?sql7=true&database=LojaVirtual<br>
 * [USER]  USR_LOJAVIRTUAL<br>
 * [PASSWORD]  USR_LOJAVIRTUAL<br>
 * [NOME_DO_COMANDO_SQL_1]<br>
 *   comando sql<br>
 * [NOME_DO_COMANDO_SQL_1]<br>
 *   comando sq2<br>
 * [NOME_DO_COMANDO_SQL_n]<br>
 *   comando n<br>
 *
 * Obs: O nome do arquivo contendo as definies sql deve ser passado no
 * construtor, assim qualquer interao com o banco de dados ser atravs
 * das informaes contidas nesse arquivo.
 */

public class Database {

  /* Conexo com o banco de dados */
  private Connection connection;
  /* Hashtable para armazenar todas as entradas do arquivo definies */
  private Hashtable template;

  /**
   * Contrutor para inicializar o objeto Database com as definies contidas
   * no arquivo de configurao
   * @param sqlFileName Nome do arquivo de configurao
   */
  public Database (String sqlFileName)
    throws Exception {
    String sqlFile = getTextFile(sqlFileName);
    template = new Hashtable();
    loadTemplate(template, sqlFile);
    String driver = (String) template.get("DRIVER");
    String url      = (String) template.get("URL");
    String user     = (String) template.get("USER");
    String password = (String) template.get("PASSWORD");
    if ( driver == null ) {
      String msg = "Definio [DRIVER] ou seu contedo no enconttrado no " +
                   "arquivo de configurao.";
      throw new Exception (msg);
    }
    if ( url == null ) {
      String msg = "Definio [URL] ou seu contedo no enconttrado no " +
                   "arquivo de configurao.";
      throw new Exception (msg);
    }
    if ( user == null ) {
      String msg = "Definio [USER] ou seu contedo no enconttrado no " +
                   "arquivo de configurao.";
      throw new Exception (msg);
    }
    if ( password == null ) {
      String msg = "Definio [PASSWORD] ou seu contedo no enconttrado no " +
                   "arquivo de configurao.";
      throw new Exception (msg);
    }
    try {
      Class.forName(driver);
      connection = DriverManager.getConnection(url, user, password);
    } catch (ClassNotFoundException e) {
      String msg = "Driver JDBC no encontrado : " + driver;
      throw new Exception (msg);
    } catch (SQLException e) {
      String msg = "Erro na conexo ao banco de dados : \n" +
                   "url     = " + url + "\n" +
                   "usurio = " + user + "\n" +
                   "senha   = " + password;
      throw new Exception (msg);
    } catch (Exception e) {
      String msg = "Erro no esperado : " + e.getMessage();
      throw new Exception (msg);
    }
  }

  /**
   * Mtodo para executar um comando sql de consulta
   * @param sql Consulta sql a ser executada
   * @return retorno da execuo da consulta solicitada
   */
  public final ResultSet executeQuery (String sql ) throws  Exception {
    Statement statement = connection.createStatement();
    return statement.executeQuery(sql);
  }

  /**
   * Mtodo para executar um comando SQL de atualizao
   * @param sql comando SQL de atualizao a ser executado
   * @return int quantidade de linhas afetadas pela execuo do comando SQL
   * solicitado
   */
  public final int executeUpdate (String sql ) throws  Exception {
    Statement statement = connection.createStatement();
    return statement.executeUpdate(sql);
  }

  /**
   * Mtodo para retornar um PreparedStatement j carregado com o comando sql
   * contido no arquivo de configurao sobre a template informada.<br>
   * Exemplo : <br>
   * [TEMPLATE_NAME]<br>
   *   comando sql<br>
   * Assim, basta setar os parametros e em seguida executar o PreparedStatement.
   * @return objeto Statement
   */
  public final PreparedStatement getPreparedStatement (String templateName)
    throws  Exception  {
    String template = this.getTemplate(templateName);
    return connection.prepareStatement(template);
  }

  /**
   * Mtodo para finalize, para ser usado pelo Garbage Collector, finalizando a
   * conexo
   */
  public void finalize () throws Exception {
    connection.close();
  }

  /**
   * Mtodo para retornar o contedo de um template do arquivo de configurao
   * @param label Nome do template
   * @return contedo do template
   */
  public final String getTemplate (String label) {
    return (String) template.get(label);
  }

  /**
   * Mtodo para carregar o arquivo de template do disco para um Hashtable
   **/
  public final void loadTemplate (Hashtable template, String sqlFile)
    throws Exception {
    int start = 0;
    int end = sqlFile.length();
    while ( start < end ) {
      int startLabel = sqlFile.indexOf("[", start);
      int endLabel = sqlFile.indexOf("]", startLabel);
      String label = sqlFile.substring(startLabel + 1, endLabel);
      int startText = endLabel + 1;
      int endText = sqlFile.indexOf("[", startText);
      if ( endText == -1 ) {
        endText = end;
      }
      String text = sqlFile.substring(startText, endText);
      template.put(label, text.trim());
      start = endText;
    }
  }

  /**
   * Mtodo utilitrio para ler o contedo de um arquivo texto
   * @param nome do arquivo texto
   * @return contedo do arquivo texto
   */
  public final static String getTextFile (String fileName)
    throws Exception {
    Reader reader = null;
    try {
      if ( reader == null )
        reader = new FileReader( fileName );
    } catch (FileNotFoundException e) {
      throw new Exception( "Arquivo no encontrado" + fileName);
    }
    try {
      StringBuffer content = new StringBuffer();
      BufferedReader is = new BufferedReader(reader);
      String linha = is.readLine();
      while ( linha != null ) {
        content.append( linha ).append( "\r\n");
        linha = is.readLine();
      }
      return content.toString();
    } catch (IOException e) {
      throw new Exception( "Erro de leitura do arquivo : " + fileName);
    } finally {
      try {
        reader.close();
      } catch(IOException e) {
        throw new Exception( "Erro fechando arquivo : " + fileName);
      }
    }
  }
}